#!/usr/bin/python
import os
import subprocess
import tempfile
from amara import bindery
from amara.xslt import transform
from Cheetah.Template import Template
from pkg_resources import resource_filename, resource_listdir
from time import time
import treecutter.constants as const
from treecutter.tools import mkdir_p

class Page():
    """Class representing a version of a webpage"""
    def __init__(self,link,page):
        self._link = link
        self._file = page[1]
        self._lang = page[0]
        self._doc = None
        self._resources = []
        self._title = None
        self._menu = None
        self._rendered_article = None

    def language(self):
        return self._lang

    def resources(self):
        return set(self._resources)

    def menu(self):
        return self._menu

    def set_article(self,art):
        self._rendered_article = art

    def prepare(self):
        self._doc = bindery.parse(self._file, prefixes=const.PREFIXES)
        if self._doc.xml_select(u'/db:article/db:info/db:title'):
            self._title = unicode(self._doc.article.info.title)
        if self._doc.xml_select(u'/db:article/db:info/db:titleabbrev'):
            self._menu = unicode(self._doc.article.info.titleabbrev)

        dirname = os.path.dirname(self._file)
        cwd = os.getcwd()
        code  = self._doc.xml_select(u"//xi:include[@parse='text']")
        if code:
            for c in code:
                (p, ext) = os.path.splitext(c.href)
                if ext in const.valid_scripts:
                    exe = []
                    script = os.path.join(os.path.abspath(dirname)+'/'+c.href)
                    if os.path.isfile(script):
                        exe.append(script)
                    else:
                        if c.href in resource_listdir('xinclude', ''):
                            script = resource_filename('xinclude', c.href)
                            exe.append(script)
                        else:
                            print "Script "+c.href+" in "+self._file+" missing"
                    if c.xml_select(u"//xi:include[@accept-language]"):
                        alang = c.xml_attributes[None, "accept-language"]
                        exe.append("lang="+alang)
                    if c.xml_select(u"//xi:include[@xpointer]"):
                        exe.append("xptr="+c.xpointer)
                    print "  executing %15s" % (c.href),
                    ts = time()
                    os.chdir(dirname)
                    xml = subprocess.Popen(exe,stdout=subprocess.PIPE)
                    os.chdir(cwd)
                    xmlblock = str(xml.stdout.read())
                    te = time()
                    print " [%5.2f s]  (%s)" % (round(te-ts,2),c.xpointer)
                    xstr = bindery.parse(xmlblock)
                    idp = c.xml_index_on_parent
                    for x in xstr.xml_children:
                        c.xml_parent.xml_insert(idp,x)
                        c.xml_parent.xml_remove(c)

        for r in self._doc.xml_select(u"//db:link[@xl:href]"):
            rf = os.path.join(dirname,r.href)
            if os.path.isfile(rf):
                self._resources.append(rf)
        for i in self._doc.xml_select(u"//db:imagedata[@fileref]"):
            im = os.path.join(dirname,i.fileref)
            if os.path.isfile(im):
                self._resources.append(im)
        for i in self._doc.xml_select(u"//html:form[@action]"):
            pyscript = re.split('\.py',i.action,1)[0]+'.py'
            im = os.path.join(dirname,pyscript)
            if os.path.isfile(im):
                self._resources.append(im)

    def render(self, style):
        #  amara can not handle the docbook stylesheets
        #  xmlarticle = transform(doc,style_xslt)
        cwd = os.getcwd()
        dirname = os.path.dirname(self._file)
        os.chdir(dirname)
        infile  = os.path.basename(tempfile.mktemp())
        outfile = tempfile.mktemp()
        tfi = open(infile,'w')
        tfi.write(self._doc.xml_encode(omit_xml_declaration=True))
        tfi.close()
#  cmd = ["saxon-xslt-xinclude","-o",outfile,infile,style_xslt]
        cmd = ["xsltproc","--xinclude","--output",outfile,style+"docbook.xsl",infile]
        retcode = subprocess.call(cmd)
        if retcode:
            print 'Error: '+' '.join(cmd)+' Returncode ['+str(retcode)+']'
        tfo = open(outfile,'r')
        self._rendered_article = tfo.read()
        tfo.close()
        os.remove(infile)
        os.remove(outfile)
        os.chdir(cwd)

    def template(self,sitemap,style,tdir):
        htmlmenu =  sitemap.gen_menu(self._lang,None,"menu")
        levelmenu = sitemap.gen_menu(self._lang,self,"tree")
        langmenu = sitemap.lang_menu(self._lang,self._link)
        template = Template(file=style+'index.'+self._lang+'.html.tmpl',
                            searchList=[{'title':self._title},
                                        {'menu':htmlmenu},
                                        {'article':self._rendered_article},
                                        {'levelmenu':levelmenu},
                                        {'langmenu':langmenu}])
        outfile = tdir+'html'.join(self._file.rsplit('xml',1))
        mkdir_p(os.path.dirname(outfile))
        out = open(outfile, 'w')
        out.write(str(template))
        out.close()
